﻿//'*****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************/

// -------------------------------------------------------------------
// FVX Sample Program
// メインダイアログ
//
// 作成者 : R.Nagashima
//
// 履歴 : 2007/03/08 初版
// -------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
//FVXデータ構造の使用宣言
using FVDATASLib;

namespace RSearch
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

		//処理範囲
		FvRectangle m_ExeRegion = new FvRectangle();
		// ビデオデバイスがオープンされているか
		private bool m_blnVideoOpened = false;
		// ビデオ設定ファイル名（フルパス）
		private string m_strVideoConfFilePath = "";
		// 初期ディレクトリ
		private string m_strDirectory;
		//エッジ表示
		frmPtnDisp m_frmPtnDisp = new frmPtnDisp();

		
        private void frmMain_Load(object sender, EventArgs e)
        {
			int IntErr;
			frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
			frmLicense fLicense = new frmLicense();

			//パタンエッジ表示用ダイアログの親を設定
			m_frmPtnDisp.Owner = this;


			// 変数初期化
			m_blnVideoOpened = false;
			btnGrab.Enabled = false;

			//パラメータ設定用コントロールの初期化
			cmbPolarity.Items.Insert(0, "同一極性");	//SAME_TYPE 登録パタンと同じ極性のパタンのみをサーチします。
			cmbPolarity.Items.Insert(1, "逆極性");		//REVERSE_TYPE 登録パタンと白黒反転したパタンのみをサーチします。
			cmbPolarity.Items.Insert(2, "両極性");		//EITHER_TYPE 登録パタンと同じ極性のパタンと白黒反転したパタンの両方をサーチします。
			cmbPolarity.SelectedIndex = 0;				//又、１つのワークが部分的に白黒反転している場合もサーチします。

			cmbPatternStandard.Items.Insert(0, "手動設定"); //OFF_TYPE 回答基準点の自動設定は行いません。
			cmbPatternStandard.Items.Insert (1,"自動設定"); //ON_TYPE 回答基準点の自動設定を行います。
			cmbPatternStandard.SelectedIndex = 0;

			cmbMarkType.Items.Insert(0, "一般形");				//GNRL_TYPE 一般形
			cmbMarkType.Items.Insert(1, "矩形（枠無し）");		//RECTANGLE1_TYPE 矩形（枠無）
			cmbMarkType.Items.Insert(2, "矩形（枠有り）");		//RECTANGLE2_TYPE 矩形（枠有り）
			cmbMarkType.Items.Insert(3, "円形（枠無し）= 円盤");//CERCLE1_TYPE 円形（枠無）= 円盤
			cmbMarkType.Items.Insert(4, "円形（枠有り）= 円環");//CIRCLE2_TYPE 円形(枠有) = 円環
			cmbMarkType.Items.Insert(5, "十字形（枠無し）");	//CROSS1_TYPE 十字形（枠無）
			cmbMarkType.Items.Insert(6, "十字形（枠有り）");	//CROSS2_TYPE 十字形（枠有り）
			cmbMarkType.SelectedIndex = 0;

			cmbMarkColor.Items.Insert(0, "黒");		//BLACK_COLOR 黒
			cmbMarkColor.Items.Insert(1, "白");		//WHITE_COLOR 白
			cmbMarkColor.Items.Insert(2, "自動");	//AUTO_COLOR 自動決定
			cmbMarkColor.SelectedIndex = 2;

			cmbMarkProj.Items.Add("0");          //0～32(8で割れる倍数)です。-1以外では0と設定して下さい。
			cmbMarkProj.Items.Add("-1");         //自動で設定します。(0と比較した場合、精度が落ちる可能性があります。)
			cmbMarkProj.SelectedIndex = 0;

			cmbCrsMethod.Items.Add("0");         //比較的大きなサイズの矩形、十字、円形の対象物に有効なアルゴリズムです。
			cmbCrsMethod.Items.Add("1");         //小さな対象物や細長い対象物など、0で対応できない対象物もサーチすることが出来ます。
			cmbCrsMethod.SelectedIndex = 0;

			cmbEdgeMethod.Items.Add("0");        //標準的なエッジ手法です。
			cmbEdgeMethod.Items.Add("1");		 //細線パタンに有効なエッジ取得手法です。
			cmbEdgeMethod.Items.Add("2");        //ノイズや緩やかな濃淡変化からエッジを取得しないようなエッジ取得手法です。
			cmbEdgeMethod.SelectedIndex = 0;

			cmbAssemblyNum.Items.Add("1");      //1を設定する際は、CrsMethodプロパティが0のときのみ有効です。
			cmbAssemblyNum.Items.Add("5");
			cmbAssemblyNum.SelectedIndex = 1;

			//画面の生成
			axFvxDisplayEx1.CreateDisplay();

			//処理範囲の初期値設定(画像の全面に設定）
			m_ExeRegion.xs = -1;
			m_ExeRegion.xe = -1;
			m_ExeRegion.ys = -1;
			m_ExeRegion.ye = -1;

			//作業ディレクトリ退避
			m_strDirectory = Directory.GetCurrentDirectory();

			//使用許諾の確認
			if (fLicense.ShowDialog() == DialogResult.OK)
			{
				//ビデオ設定ファイルの選択
				if (fVideoConf.ShowDialog() == DialogResult.OK)
					m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

				if (fVideoConf.chkNotUseCamera.Checked == false)
				{
					//ビデオ設定ファイルをコンポーネントに設定
					axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

					//デバイスオープン
					try
					{
						axFvxVideo1.OpenDevice();
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
					//このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
					//本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
					//お使いのキャプチャボードに合わせて変更してください。
					Thread.Sleep(350);

					//GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
					//あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
					//（取り込みサイズとImageのサイズが違う場合エラーになります）
					axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
					try
					{
						// 同期取込
						axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//ビデオデバイスの状態を保存
					m_blnVideoOpened = true;
					btnGrab.Enabled = true;

					//画像を表示
					axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
				}
				//サーチパタンとサーチ対象画像の全エッジを取得する設定
				axFvxFormRotator1.SetDetailedFlag(FVXCOMFORMROTATORLib.FORMROTATOR_FLAG_SWITCH.FORMROTATOR_ON);
			}
			else
			{
				//終了
				Application.Exit();
				return;
			}

        }

		//終了時の処理
		private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (m_blnVideoOpened)
			{
				// ビデオデバイスがオープンされていた場合はクローズします｡
				axFvxVideo1.CloseDevice();
			}
		}

		//カメラ画像取り込み
		private void btnGrab_Click(object sender, EventArgs e)
		{
			//画像ファイル読み込みによって、画像サイズが初期化時から
			//変更されている可能性があるため、ChangeImageSizeを行います。
			//画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
			axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

			try
			{
				//同期取込
				axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
			}
			catch (Exception)
			{
				int IntErr;
				axFvxVideo1.GetErrReport(out IntErr);
				MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
			}

			//取り込んだ画像を画面に表示
			axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
		}

		//画像ファイル読込
		private void btnLoadImage_Click(object sender, EventArgs e)
		{
			// ダイアログにて画像ファイルを選択します｡
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			openFileDialog1.DefaultExt = "*.bmp";
			openFileDialog1.ShowDialog();

			if (openFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを用いて、
				//ファイルからImageに画像を読み込みます｡
				axFvxFileAccess1.FilePath = openFileDialog1.FileName;

				try
				{
					axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
					return;
				}

				//画像タイプチェック
				if ((axFvxImage1.BitPerPixel != 8) || (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray))
				{
					MessageBox.Show("8bitグレイの画像ではありません｡");
					axFvxImage1.BitPerPixel = 8;
					axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
					axFvxImage1.ImageWidth = 512;
					axFvxImage1.ImageHeight = 480;
				}
				axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			}
		}

		//画像ファイル保存
		private void btnSaveImage_Click(object sender, EventArgs e)
		{
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			saveFileDialog1.DefaultExt = "*.bmp";
			saveFileDialog1.ShowDialog();

			if (saveFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを使って
				//Imageをファイルに保存します｡
				axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
				try
				{
					axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
					return;
				}
			}

		}
		
		//処理範囲設定
		private void btnExeRegion_Click(object sender, EventArgs e)
		{
			int intOK = 0;
			double dblTemp = 0.0;

			axFvxROI1.SetRectangle(m_ExeRegion);

			//FvxROIを使って処理範囲選択ダイアログを出します。
			axFvxROI1.UsePoint = false;
			axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out intOK);

			if ((intOK == 1) && (axFvxROI1.CheckRectangle == 1))
			{
				axFvxROI1.GetRectangle(m_ExeRegion);
				// 始点と終点が逆になっていないかチェックします｡
				// FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				// 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if (m_ExeRegion.xe < m_ExeRegion.xs)
				{
					dblTemp = m_ExeRegion.xs;
					m_ExeRegion.xs = m_ExeRegion.xe;
					m_ExeRegion.xe = dblTemp;
				}
				if (m_ExeRegion.ye < m_ExeRegion.ys)
				{
					dblTemp = m_ExeRegion.ys;
					m_ExeRegion.ys = m_ExeRegion.ye;
					m_ExeRegion.ye = dblTemp;
				}
			}
		}

		//パタン登録
		private void btnSetPattern_Click(object sender, EventArgs e)
		{
			FvRectangle pRect = new FvRectangle();//パタン領域
			FvPoint pPoint = new FvPoint();//パタン基準点

			int IntOK = 0;
			double dblTemp = 0;

			axFvxROI1.UsePoint = true;
			
			// 登録時の初期矩形を設定
			pRect.xs = 100;
			pRect.ys = 100;
			pRect.xe = 200;
			pRect.ye = 200;
			axFvxROI1.SetRectangle(pRect);

			try
			{
				//FvxROIを使って処理範囲選択ダイアログを出します。
				axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

				if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
				{
					axFvxROI1.GetRectangle(pRect);

					//始点と終点が逆になっていないかチェックします｡
					//FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
					//逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
					if (pRect.xe < pRect.xs)
					{
						dblTemp = pRect.xs;
						pRect.xs = pRect.xe;
						pRect.xe = dblTemp;
					}
					if (pRect.ye < pRect.ys)
					{
						dblTemp = pRect.ys;
						pRect.ys = pRect.ye;
						pRect.ye = dblTemp;
					}

					if (axFvxROI1.CheckPoint == 1)
					{
						axFvxROI1.GetPoint(pPoint);
					}
					else
					{
						MessageBox.Show("パターン基準点が指定されませんでした。\r\nパターン矩形の中心を基準点として登録します｡", "パターン登録", MessageBoxButtons.OK, MessageBoxIcon.Error);
						pPoint.x = (pRect.xs + pRect.xe) / 2;
						pPoint.y = (pRect.ys + pRect.ye) / 2;
					}
					axFvxPattern1.MakePattern(axFvxImage1.GetOcx(), pRect, pPoint);
				}
			}
			catch (Exception)
			{
				int IntErr;
				axFvxPattern1.GetErrReport(out IntErr);
				MessageBox.Show("パターン登録失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
		}

		//パタンのマスク設定
		private void btnSetPatternMask_Click(object sender, EventArgs e)
		{
			axFvxPattern1.DefMask(-1, -1);
		}

		//パタンの表示
		private void btnDipsPattern_Click(object sender, EventArgs e)
		{
			axFvxPattern1.Disp();
		}

		//パタンの読込
		private void btnLoadPattern_Click(object sender, EventArgs e)
		{
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "pattern file(*.pat)|*.pat";
			openFileDialog1.DefaultExt = "*.pat";
			openFileDialog1.ShowDialog();

			if (openFileDialog1.FileName.Length > 0)
			{
				try
				{
					axFvxPattern1.LoadPattrenFileName = openFileDialog1.FileName;
					axFvxPattern1.Load();
				}
				catch (Exception)
				{
					int IntErr;
					axFvxPattern1.GetErrReport(out IntErr);
					MessageBox.Show("パターン読込失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
			}
		}

		//パタンの保存
		private void btnSavePattern_Click(object sender, EventArgs e)
		{
			//パタンファイルの拡張子は特に決められていませんが、
			//"pat"を使用されることを推奨します。
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "pattern file(*.pat)|*.pat";
			saveFileDialog1.DefaultExt = "*.pat";
			saveFileDialog1.ShowDialog();
			if (saveFileDialog1.FileName.Length > 0)
			{
				try
				{
					axFvxPattern1.SavePattrenFileName = saveFileDialog1.FileName;
					axFvxPattern1.Save();
				}
				catch (Exception)
				{
					int IntErr;
					axFvxPattern1.GetErrReport(out IntErr);
					MessageBox.Show("パターン保存失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
			}
		}

		//入力された各パラメータの適用
		public void ApplyChanges()
		{
			int Temp;
			
			//サーチ個数
			if (int.TryParse(txtSearchNum.Text ,out Temp) == false || Temp < 1)
			{
				MessageBox.Show("個数は 1 以上を指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.SearchNum = Temp;

			//判定スコア
			if (int.TryParse(txtThreshold.Text, out Temp) == false || Temp < 10 || Temp > 100)
			{
				MessageBox.Show("判定閾値は 10～100 の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.ScoreThresh = Temp;

			int Min,Max;
			//回転角範囲
	        //-360　≦ AngleMin ≦ 360,　 0 ≦ AngleMax ≦ 360
	        //AngleMin ≦ AngleMax,　　   AngleMax - AngleMin ≦ 360
			if(int.TryParse(txtAngleMin.Text,out Min) == false || Min < -360 || 360 < Min)
			{
				MessageBox.Show("最小角度は -360°～ 360°の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if(int.TryParse(txtAngleMax.Text,out Max) == false || Max < 0 || 360 < Max)
			{
				MessageBox.Show("最大角度は 0°～ 360°の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (Min > Max)
			{
				MessageBox.Show("(最小角度)≦(最大角度)となるように指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if((Max - Min) > 360)
			{
				MessageBox.Show("( (最大角度) - (最小角度) ) ≦ 360 となるように指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.AngleMin = Min;
			axFvxFormRotator1.AngleMax = Max;

			//スケール範囲
			//0 < ScaleMin, ScaleMin ≦ ScaleMax
			if(int.TryParse(txtScaleMin.Text,out Min) == false || Min < 1)
			{
				MessageBox.Show("最小スケール値は １ 以上を指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if(int.TryParse(txtScaleMax.Text,out Max) == false || Max < 1)
			{
				MessageBox.Show("最大スケール値は １ 以上を指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if(Min > Max)
			{
				MessageBox.Show("(最小スケール値)≦(最大スケール値)となるように指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.ScaleMin = Min;
			axFvxFormRotator1.ScaleMax = Max;

			//エッジ極性
			axFvxFormRotator1.Polarity = (FVXCOMFORMROTATORLib.POLARITY_TYPE )cmbPolarity.SelectedIndex;

			//回答基準点設定
			axFvxFormRotator1.StdPtnAuto = (FVXCOMFORMROTATORLib.STD_AUTO_TYPE)cmbPatternStandard.SelectedIndex;

			//マーク形状
			axFvxFormRotator1.MarkType = (FVXCOMFORMROTATORLib.MARK_PARAMETER_TYPE)cmbMarkType.SelectedIndex;

			//マーク色
			axFvxFormRotator1.MarkColor = (FVXCOMFORMROTATORLib.COLOR_PARAMETER_TYPE)cmbMarkColor.SelectedIndex;

			//タイムアウト
			if(int.TryParse(txtTimeOut.Text,out Temp) == false || Temp < 0)
			{
				MessageBox.Show("タイムアウトは ０ 以上を指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.TimeOut = Temp;
			
			//処理範囲設定
			axFvxFormRotator1.SetInspectArea(m_ExeRegion);

			//不可欠領域スコア
			if(int.TryParse(txtEssThresh.Text,out Temp) == false || Temp < 1 || Temp > 100)
			{
				MessageBox.Show("不可欠領域スコアは 1～100 の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.EssThresh = Temp;

			//射影分割数
			if( cmbMarkProj.Text == "0")
				axFvxFormRotator1.MarkProj = 0;
			else
				axFvxFormRotator1.MarkProj = -1;

			//満点しきい値・零点しきい値
			//0 ≦ MatchFull < MatchZero ≦ 90
			if(int.TryParse(txtMatchFull.Text,out Min) == false || Min < 0 || Min > 89)
			{
				MessageBox.Show("満点しきい値は 0～89 の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if(int.TryParse(txtMatchZero.Text,out Max) == false || Max < 1 || Max > 90)
			{
				MessageBox.Show("零点しきい値は 1～90 の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if(Min >= Max)
			{
				MessageBox.Show("(満点しきい値) < (零点しきい値)となるように指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.MatchFull = Min;
			axFvxFormRotator1.MatchZero = Max;

			//コントラストしきい値
	        //0 ≦ MatchRate
			if(int.TryParse(txtMatchRate.Text,out Temp) == false || Temp < 0)
			{
				MessageBox.Show("コントラストしきい値は ０ 以上で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			axFvxFormRotator1.MatchRate = Temp;

			//探索アルゴリズム手法
			axFvxFormRotator1.CrsMethod = cmbCrsMethod.SelectedIndex;

			//エッジ取得しきい値
			if(chkEdgeThresholdAuto.Checked)
				axFvxFormRotator1.EdgeThreshold = -1;
			else
			{
				if(int.TryParse(txtEdgeThreshold.Text,out Temp) == false || Temp < 0 || Temp > 255)
				{
					MessageBox.Show("エッジ取得しきい値は 0～255 の範囲で指定して下さい", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				axFvxFormRotator1.EdgeThreshold = Temp;
			}

			//エッジ取得方法
			axFvxFormRotator1.EdgeMethod = cmbEdgeMethod.SelectedIndex;

			//アセンブリ数
			if( cmbAssemblyNum.Text == "1")
			{
				if(axFvxFormRotator1.CrsMethod != 0)
				{
					MessageBox.Show("アセンブリ数 1 は探索アルゴリズム手法が 0 の時のみ有効です", "パラメータエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				axFvxFormRotator1.AssemblyNum = 1;
			}
			else
				axFvxFormRotator1.AssemblyNum = 5;
		}

		
		//実行
		private void btnExecute_Click(object sender, EventArgs e)
		{
			FvSearchResultGroup SearchResultGroup = new FvSearchResultGroup();
					
			double dPatternSetTime;
			double dSearchTime;
			int Count;

			btnExecute.Enabled = false;

			axFvxDisplayEx1.ClearOverlay(-1);

			//パラメータ設定
			ApplyChanges();

			try
			{
				axFvxPerformanceCounter1.StartCount();
				axFvxFormRotator1.SetPattern(axFvxPattern1.GetOcx());//サーチパタン設定
				axFvxPerformanceCounter1.StopCount(out dPatternSetTime);
			}
			catch (Exception)
			{
				int intErr;
				axFvxFormRotator1.GetErrReport(out intErr);
				if (intErr != 0)
				{
					MessageBox.Show(GetFormRotatorErrMesg(intErr), "パタン登録エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
					btnExecute.Enabled = true;
				}
				else{
					axFvxFormRotator1.GetPatternError(out intErr); //詳細エラー取得
					MessageBox.Show(GetFormRotatorErrMesg(intErr), "パタン登録エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
					btnExecute.Enabled = true;
				}
				return;
			}
		
			try
			{
				axFvxPerformanceCounter1.StartCount();
				axFvxFormRotator1.Execute(axFvxImage1.GetOcx(), out Count);//サーチ実行
				axFvxPerformanceCounter1.StopCount(out dSearchTime);
			}
			catch (Exception)
			{
				int intErr;
				axFvxFormRotator1.GetErrReport(out intErr);
				MessageBox.Show(GetFormRotatorErrMesg(intErr), "サーチエラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				btnExecute.Enabled = true;
				return;
			}

			//処理範囲を青で表示
			axFvxDisplayEx1.PenColor = Color.Blue;
			axFvxDisplayEx1.DrawData(0, m_ExeRegion);

			txtResult.Text = string.Format("処理時間 \r\n"+
											"SetPattern : {0:0.00}ms\r\n"+
											"Execute : {1:0.00}ms\r\n"+
											"サーチ個数 : {2:0}\r\n" + "\r\n", 
											dPatternSetTime,
											dSearchTime,
											Count);
			//結果を取得して赤で表示
			//(xxResultGroupにて結果を受け取る場合には、第一引数は無視されます。)
			axFvxFormRotator1.GetResult(0, SearchResultGroup);
			axFvxDisplayEx1.PenColor = Color.Red;
			axFvxDisplayEx1.DrawData(0,SearchResultGroup);

			for (int i = 0; i <= SearchResultGroup.Count - 1; i++ )
			{
				txtResult.Text = txtResult.Text + string.Format("No,        X,        Y,     角度,     スケール,     スコア,     不可欠部分スコア\r\n"+
																"{0:0},       {1:0.0},  {2:0.0},    {3:0.0},       {4:0.0},       {5:0.0},       {6:0.0}\r\n",
												i, SearchResultGroup.Item2(i).x, SearchResultGroup.Item2(i).y,
												SearchResultGroup.Item2(i).angle, SearchResultGroup.Item2(i).magnification,
												SearchResultGroup.Item2(i).score, SearchResultGroup.Item2(i).ess_score);
			}

			if(chkDetailedFlag.Checked == true)
				EdgeDisp(0,1);

			btnExecute.Enabled = true;

		}

		//エラー取得
		private string GetFormRotatorErrMesg(int intErr)
		{
			string ErrMesg;

			switch (intErr)
			{
				case 5001:
					ErrMesg = "パタンが存在していません。";
					break;
				case 5005:
					ErrMesg = "パラメータが違います。";
					break;
				case 5006:
					ErrMesg = "引数の型が違います。";
					break;
				case 5007:
					ErrMesg = "パタン画像が一定のため、登録できません。";
					break;
				case 5008:
					ErrMesg = "パタンとマスクのサイズが一致していません。";
					break;
				case 5009:
					ErrMesg = "パラメータにNULL値が指定されました。";
					break;
				case 5010:
					ErrMesg = "メモリの確保に失敗しました。";
					break;
				case 5020:
					ErrMesg = "コントロールのオープン処理に失敗しました。";
					break;
				case 5022:
					ErrMesg = "パタンが指定されていません。";
					break;
				case 5030:
					ErrMesg = "登録パタンが存在しません。";
					break;
				case 5031:
					ErrMesg = "登録パタンが２重に定義されています。";
					break;
				case 5033:
					ErrMesg = "登録パタンの情報が足りません。";
					break;
				case 5034:
					ErrMesg = "計測画像のBPPが８ではありません。";
					break;
				case 5035:
					ErrMesg = "タイムアウトエラーです。";
					break;
				case 5040:
					ErrMesg = "対象物の回転角を決定することができません。";
					break;
				case 5041:
					ErrMesg = "画像中のエッジが不足しています。";
					break;
				case 5042:
					ErrMesg = "パタン領域が小さすぎるか、エッジ情報が足りません。";
					break;
				case 5043:
					ErrMesg = "サーチに適した輪郭形状が見つかりませんでした。";
					break;
				case 5044:
					ErrMesg = "指定されたマーク形状が見つかりませんでした。";
					break;
				case 5045:
					ErrMesg = "指定されたマーク形状が見つかりませんでした。";
					break;
				case 5099:
					ErrMesg = "原因不明のエラーです。";
					break;
				case 9502:
					ErrMesg = "サポートされていないイメージタイプです。";
					break;
				case 9505:
					ErrMesg = "パラメータが違います。";
					break;
				case 9506:
					ErrMesg = "引数の型が違います。";
					break;
				case 9510:
					ErrMesg = "メモリの確保に失敗しました。";
					break;
				case 9531:
					ErrMesg = "パスの指定が間違っています。";
					break;
				case 9533:
					ErrMesg = "パターンが存在しません。";
					break;
				case 9540:
					ErrMesg = "パタンが存在しません。";
					break;
				default: 
					ErrMesg = "不明なエラー";
					break;
			}
			return (ErrMesg);
		}
		

		//パターンのエッジ表示
		private void btnDispPatternEdge_Click(object sender, EventArgs e)
		{
			try
			{
				axFvxFormRotator1.SetPattern(axFvxPattern1.GetOcx());
			}
			catch (Exception)
			{
				int IntErr;
				axFvxFormRotator1.GetErrReport(out IntErr);
				MessageBox.Show("パターン保存失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			m_frmPtnDisp.ShowDialog();
		}
		
		//エッジ描画
		public void EdgeDisp(int Kind, int Mode)
		{        //intKind  0:画像 1:パタン   intMode 0: Clear 1: Draw 2:Clear & Draw

			FvPointGroup pPointDataG = new FvPointGroup();
			FvPoint pPointData = new FvPoint();
	
			try{
				if( Mode == 1 || Mode == 2)
				{
					//パラメータ設定
					ApplyChanges();

					if(Kind == 0)	//サーチ画像エッジ表示
					{
						axFvxDisplayEx1.ClearOverlay(1);
					
						//処理範囲表示色を青に
						axFvxDisplayEx1.PenColor = Color.Blue;
						axFvxDisplayEx1.DrawData(1,m_ExeRegion);
						
						//エッジ表示色を緑に
						axFvxDisplayEx1.PenColor = Color.Lime;

						//画像のエッジを取得
						axFvxFormRotator1.GetDetailedEdge(pPointDataG);

						//描画
						for (int i = 0; i <= pPointDataG.Count - 1; i++)
						{
							pPointData = pPointDataG.Item2(i);
							//処理範囲が変った場合、エッジの表示位置をDisplayExに合わせる
							if(axFvxFormRotator1.InpArea_SX != -1)
								pPointData.x += axFvxFormRotator1.InpArea_SX; 
							if(axFvxFormRotator1.InpArea_SY != -1)
								pPointData.y += axFvxFormRotator1.InpArea_SY;

							axFvxDisplayEx1.DrawData(1, pPointData);
						}
					}
					else　//パタンエッジ表示
					{
						if(Mode == 2)
							m_frmPtnDisp.axFvxDisplayEx1.ClearOverlay(1);

						//エッジ表示色を緑に
						m_frmPtnDisp.axFvxDisplayEx1.PenColor = Color.Lime;

						//パタンのエッジを取得
						axFvxFormRotator1.GetMatchingEdge(pPointDataG);

						//エッジ描画
						m_frmPtnDisp.axFvxDisplayEx1.DrawData(0, pPointDataG);

						//中心位置表示(未完成)
						m_frmPtnDisp.PatanCenterMarkDisp();
					}
				}
				else if(Kind == 0) axFvxDisplayEx1.ClearOverlay(-1);
				else m_frmPtnDisp.axFvxDisplayEx1.ClearOverlay(-1);
			}
			catch(Exception)
			{
				int Err;
				axFvxFormRotator1.GetErrReport(out Err);
				MessageBox.Show("エッジ表示失敗 ErrCode = " + Err.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}


		//オーバーレイの消去
		private void button1_Click(object sender, EventArgs e)
		{
			axFvxDisplayEx1.ClearOverlay(-1);
		}

		//数値/Delete/BaclSpaceのみ入力できるよう制御
		private void txtSearchNum_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtThreshold_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtAngleMin_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtAngleMax_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtScaleMin_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtScaleMax_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtTimeOut_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtEssThresh_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtMatchFull_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtMatchZero_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtMatchRate_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtEdgeThreshold_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void chkEdgeThresholdAuto_CheckedChanged(object sender, EventArgs e)
		{
			if (chkEdgeThresholdAuto.Checked == true)
				txtEdgeThreshold.Enabled = false;
			else
				txtEdgeThreshold.Enabled = true;
		}
    }
}